@@ -204,7 +204,9 @@ static char ident[] =
 static char RCSid[] = 
 " @(#) deslogind.c  $Revision: 1.13 $ $Date: 95/03/08 01:13:14 $\n";
 
-static int userlogwrap = 0;
+static int userlogwrap = 0; 		/* Added by Federico audano for */
+                           		/* user login wrapper mode 	*/
+static int use_ndbm = 0;
 
 char mainProgName[PROGNAME_SIZE];
 
@@ -271,7 +273,7 @@ void eraselogin()
 {
    int res;
    res = setlogin(sname, userName, rhostName, &chldStat);
-   if (!userlogwrap) 
+   if (!userlogwrap)		/* Modified by Federico Audano */ 
     if (res < 0) {
        log("%s: WARNING--eraselogin \"%s\" failed--%s\n", 
 	  progName, sname, ERRMSG);
@@ -564,7 +566,10 @@ keyType getUserFileKey(userFile, userPhrase, noUserKey)
    /*
     * Make userfile exists and the correct key was specified
     */
-   res = getUserPhrase(userFile, passPhrase, PHRASE_SIZE, "#", userKey);
+
+   if (!use_ndbm)
+    res = getUserPhrase(userFile, passPhrase, PHRASE_SIZE, "#", userKey);
+
    if (res == -1) {
       log("%s: cannot open \"%s\" for user database\n", progName, userFile);
       return (keyType) -1;
@@ -801,7 +806,7 @@ int main(argc, argv)
       }
    }
 
-   while ((ch = getopt(argc, argv, "Ww:k:Pl:t:i:f:p:dvnc")) != EOF) switch (ch) {
+   while ((ch = getopt(argc, argv, "Ww:k:Pl:t:i:f:p:Ddvnc")) != EOF) switch (ch) {
    case 'c': 				   /* user compile'd userFile key */
       userPhrase = (char *) -1;
       break;
@@ -837,9 +842,13 @@ int main(argc, argv)
    case 'w':				/* function as wrapper for login */
       generic = optarg;	 /* name in userfile with system-wide passphrase */
       break;
-   case 'W':
-      userlogwrap = 1;
-      break;
+   case 'W':              		/* New option Federico Audano */
+      userlogwrap = 1;       		/* login wrapper with user    */
+      break;                            /* passphrase                 */
+   
+   case 'D':                             /* New option Federico Audano */
+      use_ndbm = 1;                     /* use NDBM functions for user*/
+      break;				/* database		      */
    case 'v':
       verbose++;
       break;
@@ -942,7 +951,7 @@ progName);
    /*
     * Minimize the the time the phrase must me visible in memory.
     */
-   if (userlogwrap) {
+   if (userlogwrap) {       		/* added by Federico Audano */
       generic = userName;
    }
    if (generic) {
@@ -950,10 +959,21 @@ progName);
    } else {
       chp = userName;
    }
-   res = getUserPhrase(userFile, passPhrase, PHRASE_SIZE, chp, userKey);
+
+   /* Modified by Federico Audano for NDBM */
+   if (!use_ndbm)
+   {
+    res = getUserPhrase(userFile, passPhrase, PHRASE_SIZE, chp, userKey);
+   }
+   else
+   {
+    res = ndbm_getUserPhrase(userFile,passPhrase, PHRASE_SIZE, chp, userKey);
+   }
+   /* end of modified block */
+
    if (res == 0) {
       log("%s: NOPHRASE %s@%s:%d->\"%s\"\n", 
-	 progName, ruser, rhostName, rport, chp);
+         progName, ruser, rhostName, rport, chp);
       return 1;
    }
    if (res == -1) {
@@ -962,7 +982,7 @@ progName);
    }
    if (res == -2) {
       log("%s: incorrect decryption key for user database \"%s\"\n", 
-	 progName, userFile);
+         progName, userFile);
       return 1;
    }
    key = challenge(nfd, passPhrase, loginTimeout);
@@ -1143,7 +1163,7 @@ progName);
 	  * It's required on hp-ux even for -w (wrapper) option.
 	  */
 	 
-         if (!userlogwrap) /* modifica1 */
+         if (!userlogwrap) /* Modified by Federico Audano */
          {
           res = setlogin(sname, userName, rhostName, 0);
 	  if (res < 0) {
@@ -1534,7 +1554,7 @@ progName);
        * still hasn't called setlogin by now.
        */
       chp = (char *) getlogin();
-      if(!userlogwrap) 
+      if(!userlogwrap)  		/* Modified by Federico Audano */
        if (chp == (char *) 0) {
 	  log("%s: WARNING--getlogin returned NULL\n", progName);
        } else {
